import json
import logging
import requests
from pprint import pprint
from urllib.parse import quote
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry
from pythonjsonlogger import jsonlogger

class CustomHttpHandler(logging.Handler):
    def __init__(self, url: str, token: str, silent: bool = True):
        '''
        Initializes the custom http handler
        Parameters:
            url (str): The URL that the logs will be sent to
            token (str): The Authorization token being used
            silent (bool): If False the http response and logs will be sent
                           to STDOUT for debug
        '''
        self.url = url
        self.silent = silent

        # sets up a session with the server
        self.MAX_POOLSIZE = 100
        self.session = requests.Session()
        self.session.headers.update({
            'Content-Type': 'application/json',
        })
        if token:
            self.session.headers.update({
                'Authorization': 'Bearer %s' % (token)
            })
        self.session.mount('https://', HTTPAdapter(
            max_retries=Retry(
                total=5,
                backoff_factor=0.5,
                status_forcelist=[403, 500]
            ),
        ))

        super().__init__()

    def emit(self, record):
        '''
        This function gets called when a log event gets emitted. It recieves a
        record, formats it and sends it to the url
        Parameters:
            record: a log record
        '''
        logEntry = {
            'name': record.name,
            'line': record.lineno,
            'loglevel': record.levelname,
            'message': record.getMessage(),
            'pathname': record.pathname
        }
        response = self.session.post(self.url, json=logEntry)

        if not self.silent:
            print(logEntry)

class TOPHandler(logging.StreamHandler):
    def __init__(self, service_name, version, top_tenant):
        super().__init__()
        self.add_SCS_formatter(service_name, version, top_tenant) 

    def add_SCS_formatter(self, service_name, version, top_tenant):
        formatter = jsonlogger.JsonFormatter(
            fmt='%(sky)s %(service)s %(message)s %(@timestamp)s',
            static_fields={"sky": {"top_tenant": top_tenant}, "service": {"name": service_name, "version": version}},
            timestamp='@timestamp'
        )

        # add formatter to formatter
        self.setFormatter(formatter)


