import os
import logging
import json
from logging.handlers import RotatingFileHandler
from p2.log.handlers import CustomHttpHandler, TOPHandler

class Logger():
    """
    A class to handle logging.
    Usage:
        from p2.logging import Logger

        log_obj = Logger(name=<log filename>, level=<level>
        logger = log_obj.getLogger()
    """

    def __init__(self, name: str=__name__, level=logging.INFO):
        self.log = logging.getLogger(name)
        self.log.setLevel(level)

        handler = RotatingFileHandler(
            name+'.log',
            maxBytes=1000000,
            backupCount=5
            )

        # create formatter
        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')

        # add formatter to ch
        handler.setFormatter(formatter)
        self.log.addHandler(handler)

    def addTOPHandler(self, *, service_name, version, top_tenant):
        handler = TOPHandler(service_name, version, top_tenant)
        self.log.addHandler(handler)

    def __del__(self):
        for handler in self.log.handlers:
            handler.flush()

    def getLogger(self):
        return self.log


class HttpLogger():
    """A class to send logs to the p2.store.server

    Usage:
        from p2.logging import HttpLogger

        log_obj = HttpLogger(
            level   : serverity, defaults to logging.DEBUG
            silent  : bool  if False it will anyway log to stdout
            token   : the auth token (if needed)
            name    : component name
            p2_store_url : p2.store.server url (it will default to
                            P2_STORE_URL env var
        )
        logger = log_obj.getLogger()
    """

    def __init__(self,
                 token: str = None,
                 name: str = __name__,
                 level=logging.DEBUG,
                 p2_store_url: str = None,
                 silent: bool = True):

        # create logger
        log = logging.getLogger(name)
        log.setLevel(level)

        # create formatter - this formats the log messages accordingly
        formatter = logging.Formatter(json.dumps({
            'time': '%(asctime)s',
            'name': '%(name)s',
            'pathname': '%(pathname)s',
            'line': '%(lineno)d',
            'logLevel': '%(levelname)s',
            'message': '%(message)s'
        }))

        # create a custom http logger handler
        url = p2_store_url or os.getenv('P2_STORE_URL')
        if url is None:
            raise Exception("p2.store server URL not set")

        httpHandler = CustomHttpHandler(
            url = url + '/log',
            token = token,
            silent = silent
        )

        httpHandler.setLevel(level)

        # add formatter to custom http handler
        httpHandler.setFormatter(formatter)

        # add handler to logger
        log.addHandler(httpHandler)

        self.log = log

    def addTOPHandler(self, *, service_name, version, top_tenant):
        handler = TOPHandler(service_name, version, top_tenant)
        self.log.addHandler(handler)

    def getLogger(self):
        return self.log

class TOPLogger():
    """
    A class to handle logging to TOP. service_name, top_tenant and version are mandatory and must be specified as keyword arguments.
    Note that log level must be a string
    Usage:
        from p2.logging import TOPLogger

        log_obj = TOPLogger(name=<log filename>, level=<level>, service_name=<service_name>, top_tenant=<top_tenant>, version=<version>)
        logger = log_obj.getLogger()
    """

    def __init__(self, *, service_name, version, top_tenant, name: str=__name__, level="INFO"):
        self.log = logging.getLogger(name)
        self.log.setLevel(level)
        handler = TOPHandler(service_name, version, top_tenant)
        self.log.addHandler(handler)

    def getLogger(self):
        return self.log
