import unittest
from io import StringIO
import json
import logging
from jsonschema import validate
from p2.logging import TOPLogger

class TestSCSLogging(unittest.TestCase):
    def setUp(self):
        super().setUp()
        self.service_name = "testservicename"
        self.version = "testversion"
        self.top_tenant = "testtoptenant"
        self.buffer = StringIO()
        with open('tests/schema.json') as json_file:
            self.schema = json.load(json_file)

        self.log_obj = TOPLogger(
            service_name=self.service_name,
            version=self.version,
            top_tenant=self.top_tenant,
            name="test_logger",
            level="INFO"
            )
        self.logger = self.log_obj.getLogger()

        self.test_handler = logging.StreamHandler(self.buffer) # Test handler allows capture of log output 
        self.formatter = self.logger.handlers[0].formatter
        self.test_handler.setFormatter(self.formatter)
        self.logger.handlers = []
        self.logger.addHandler(self.test_handler)

    def tearDown(self):
        super().tearDown()
        self.buffer.close()

    def test_logs_follow_scs_format(self):
        self.logger.critical("test")
        json_logs = json.loads(self.buffer.getvalue())
        validate(instance=json_logs, schema=self.schema)

    def test_correct_service_name(self):
        self.logger.critical("Does this log have the correct service name?")
        json_logs = json.loads(self.buffer.getvalue())
        self.assertEqual(json_logs["service"]["name"], self.service_name)

    def test_correct_version(self):
        self.logger.critical("Does this log have the correct version?")
        json_logs = json.loads(self.buffer.getvalue())
        self.assertEqual(json_logs["service"]["version"], self.version)
    
    def test_correct_top_tenant_present(self):
        self.logger.info("testing top tenant")
        json_logs = json.loads(self.buffer.getvalue())
        self.assertEqual(json_logs["sky"]["top_tenant"], self.top_tenant)

    def test_escape_reserved_json_characters(self):
        self.logger.error('",{"@timestamp": "hello"}, {"test": "test"\n')
        json_logs = json.loads(self.buffer.getvalue())
        validate(instance=json_logs, schema=self.schema)

    def test_must_specify_all_TOP_keyword_arguments(self):
        with self.assertRaises(TypeError):
            unspecified_keyword_arguments_TOP_logger = TOPLogger()

    def test_must_specify_TOP_arguments_as_keywords(self):
        with self.assertRaises(TypeError):
            not_using_keyword_arguments_TOP_logger = TOPLogger("servicename", "version", "toptenant")
