#!/usr/bin/env python
from typing import Type
import unittest
import logging
import os.path

from urllib.parse import quote

from p2.logging import Logger, HttpLogger
from p2.tools.prepare_db import setUpDB, tearDownDB
from p2.tools.packages import get_p2_packages
from p2.log.handlers import TOPHandler
import logging

class TestP2logging(unittest.TestCase):
    """Tests for `p2.logging` package."""

    def SetUp(self):
        setUpDB()

    def test_file_logger(self):
        log_obj = Logger(name=__name__, level=logging.INFO)
        logger = log_obj.getLogger()
        test_logger(logger, "file handler test")
        self.assertTrue(os.path.exists('./tests.test_p2_logging.log'))
        with open('./tests.test_p2_logging.log') as f:
            text = f.read()
            self.assertIn('handler', text)

    def test_http_logger(self):
        log_obj = HttpLogger(
            silent = False,
            token = '1234',
            name = 'test-http-log')
        logger = log_obj.getLogger()
        test_logger(logger, "http handler log")
        test_logger(logger, "http handler again")

    def test_http_logger_long_message(self):
        log_obj = HttpLogger(
            silent = False,
            token = '1234',
            name = 'test-http-log')
        logger = log_obj.getLogger()
        test_logger(logger, """
            This is quite a long,
            multiline message
            """)

    def test_http_logger_p2_packages(self):
        log_obj = HttpLogger(
            silent = False,
            token = '1234',
            name = 'test-http-log')
        logger = log_obj.getLogger()
        msg = "\n".join(get_p2_packages())
        test_logger(logger, msg)

    def test_add_TOP_handler_to_Http_logger(self):
        log_obj = HttpLogger(
            silent = False,
            token = '1234',
            name = 'test-http-log')
        log_obj.addTOPHandler(service_name="service_name", version="1.2.3", top_tenant="top_tenant")
        logger = log_obj.getLogger()
        for handler in logger.handlers:
            if isinstance(handler, TOPHandler):
                break # TOP handler was found so break out of for loop and else statement not run. Test passes
        else: # This runs if break statement not run meaning TOP handler not found in logger.handlers
            self.fail("TOP Handler not found. Expected that TOP Handler was added to HTTP logger")

    def test_add_TOP_handler_to_file_logger(self):
        log_obj = Logger(name=__name__, level=logging.INFO)
        log_obj.addTOPHandler(service_name="service_name", version="1.2.3", top_tenant="top_tenant")
        logger = log_obj.getLogger()
        for handler in logger.handlers:
            if isinstance(handler, TOPHandler):
                break # TOP handler was found so break out of for loop and else statement not run. Test passes
        else: # This runs if break statement not run meaning TOP handler not found in logger.handlers
            self.fail("TOP Handler not found. Expected that TOP Handler was added to file logger")

    def test_must_specify_all_TOP_handler_keyword_arguments(self):
        log_obj = Logger(name=__name__, level=logging.INFO)
        with self.assertRaises(TypeError):
            log_obj.addTOPHandler()

    def test_must_specify_all_TOP_handler_arguments_as_keywords(self):
        log_obj = Logger(name=__name__, level=logging.INFO)
        with self.assertRaises(TypeError):
            log_obj.addTOPHandler("service_name", "1.2.3", "top_tenant")
    
def test_logger(logger, msg: str):
    logger.info(msg)

